/*
Copyright Caroline et Serge COUDÉ, (04/2012) 

serge.coude@gmail.com caroline.coude@laposte.net

Ce logiciel est un programme informatique faisant partie d'une suite de 
logiciels 'TbiSuite' à destination des enseignants et des élèves usagés des 
ordinateurs et Tableau Numérique Interactif / Tableau Blanc Interactif. 

Ce logiciel est régi par la licence CeCILL v2 soumise au droit français et
respectant les principes de diffusion des logiciels libres. Vous pouvez
utiliser, modifier et/ou redistribuer ce programme sous les conditions
de la licence CeCILL v2 telle que diffusée par le CEA, le CNRS et l'INRIA 
sur le site "http://www.cecill.info".

En contrepartie de l'accessibilité au code source et des droits de copie,
de modification et de redistribution accordés par cette licence, il n'est
offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
seule une responsabilité restreinte pèse sur l'auteur du programme,  le
titulaire des droits patrimoniaux et les concédants successifs.

A cet égard  l'attention de l'utilisateur est attirée sur les risques
associés au chargement,  à l'utilisation,  à la modification et/ou au
développement et à la reproduction du logiciel par l'utilisateur étant 
donné sa spécificité de logiciel libre, qui peut le rendre complexe à 
manipuler et qui le réserve donc à des développeurs et des professionnels
avertis possédant  des  connaissances  informatiques approfondies.  Les
utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
logiciel à leurs besoins dans des conditions permettant d'assurer la
sécurité de leurs systèmes et ou de leurs données et, plus généralement, 
à l'utiliser et l'exploiter dans les mêmes conditions de sécurité. 

Le fait que vous puissiez accéder à cet en-tête signifie que vous avez 
pris connaissance de la licence CeCILL v2, et que vous en avez accepté les
termes.

*/

package net.sergecoude.TbiAdmin;

import javax.swing.*;
import java.awt.*;
import java.net.*;
import java.io.*;
import java.awt.event.*;
import java.util.*;
import java.util.jar.*;
import java.lang.reflect.*;
import java.nio.channels.*;
import com.itextpdf.text.*;
import com.itextpdf.text.pdf.*;
import net.sergecoude.TbiGeneral.*;

public class TbiAdmin extends TbiApp implements FilenameFilter {

  private final static long serialVersionUID = 1L;

  private PanelSelectClasse _pSelectClasse; 
  private PanelClasse _pClasse;
  private PanelEleve _pEleve;
  private PanelTitre _pTitre;
  private PanelSelectConfGest _pSelectConfGest;
  private PanelSelectEleves _pSelectEleves;
  private PanelSelectApplications _pSelectApplications;
  private PanelConfTbi _pConfTbi;
  private boolean _modification = false;
  private Vector<Classe> _lesClassesModifiees = null;
  private Vector<Application> _lesApplications = null;
  private ChargeurDeClasse _chargeurDePlugins = null;
  private int _typeGeneration = 0;
  
	public static int TYPE_GENERATION_PAR_APPLICATION = 1;
  public static int TYPE_GENERATION_PAR_ELEVE = 2; 
  
  public TbiAdmin(String fichierLog) {
    super(fichierLog);
    _applicationInfo = new ApplicationInfoAdmin();
    setTitle(((ApplicationInfoAdmin)_applicationInfo).getNom() + " " + ((ApplicationInfoAdmin)_applicationInfo).getVersion());
    _fichierConf = getRepertoireApplication() +"/TbiAdmin.conf";
    /*************************************
     * 0 : TbiGeneralApp.conf    
     * 1 : TbiGeneral.conf
     * 2 : TbiAdminApp.conf     
     * 3 : TbiAdmin.conf
     *************************************/
    if (!_configuration.chargeConfiguration("2", "TbiAdminApp.conf", false, false))
      System.exit(-1); 
    if (!_configuration.chargeConfiguration("3", _fichierConf, false, false))
      System.exit(-1); 
    initSuivantConfiguration(); // obligatoire !
		_pSelectClasse = new PanelSelectClasse(this);
		_pMain.add(_pSelectClasse,"SelectClasse");
 		_pClasse = new PanelClasse(this);
		_pMain.add(_pClasse,"PanelClasse");
 		_pEleve = new PanelEleve(this);
		_pMain.add(_pEleve,"PanelEleve");
		_pTitre = new PanelTitreAdmin(this);
		_pMain.add(_pTitre,"Titre");
		_pSelectConfGest = new PanelSelectConfGest(this);
		_pMain.add(_pSelectConfGest,"SelectConfGest");
		_pConfTbi = new PanelConfTbi(this);
		_pMain.add(_pConfTbi,"ConfTbi");
		_pSelectEleves = new PanelSelectEleves(this);
		_pMain.add(_pSelectEleves,"SelectEleves");
		_pSelectApplications = new PanelSelectApplications(this);
		_pMain.add(_pSelectApplications,"SelectApplications");
    getContentPane().setBackground(_cFond);
    setPanneau("Titre");
    _lesClassesModifiees = new Vector<Classe>();
    _timer = new java.util.Timer();
    _timer.schedule(new TimerTask() {
        public void run() {
            stopTimer();
            setPanneau("SelectConfGest");
        }
    }, 10000);
		setVisible(true);
  }
  
  public ChargeurDeClasse getChargeurDePlugins() {
    return _chargeurDePlugins;
  }

  @Override
  public String getRepertoireApplication() {
    return "TbiAdmin";
  } 
  
  protected void quitter() {
    if (_lesClassesModifiees.size() == 0)
      super.quitter();
    else {
      String message = "";
      if (_lesClassesModifiees.size() > 1)
        message = "Les classes ";
      else
        message = "La classe ";
      for (int c = 0; c < _lesClassesModifiees.size(); c++) {
        message += "'" + _lesClassesModifiees.elementAt(c).getNom() + "'\n";        
      }
      if (_lesClassesModifiees.size() > 1)
        message += "ont été modifiées sans être enregistrées...";
      else
        message += "a été modifiée sans être enregistrée...";
      message += "\nSouhaitez-vous quand même quitter l'application ?";     
      int rep = JOptionPane.showConfirmDialog(_pFrame, message, "Quitter",
        JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE);
      if (rep == JOptionPane.YES_OPTION) {
        super.quitter();
      }
    }
  }
  
  public String getConfiguration(String param) {
    return super.getConfiguration(param);
	}
	
  public boolean setConfiguration(String param, String valeur) {
    return super.setConfiguration(param, valeur);
	}
	
  public Classe setClasse(Classe classe) {
    super.setClasse(classe);
    return classe;
  }
  
	public Eleve setEleve(Eleve eleve) {
	  super.setEleve(eleve);
    return eleve;
  }
  
  public void editerClasse(Classe classe) {
    setClasse(classe);
		setPanneau("PanelClasse");  
  }
  
  public void enregistrerClasse(Classe classe) {
   if (!_lesClasses.contains(classe))
      _lesClasses.addElement(classe);
    modification(classe, false);
  }
  
  public void supprimerClasse(Classe classe) {
    int rep = JOptionPane.showConfirmDialog(_pFrame, "Êtes-vous sûr de vouloir retirer la classe '" + classe.getNom() + "' ?\nAttention : le fichier va être supprimé définitivement !", "Suppression",
        JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE);
    if (rep == JOptionPane.YES_OPTION) {
      File fichier = new File(classe.getFichier());
      if (fichier.exists()) {
        if (fichier.delete()) {
          modification(classe, false);
          _lesClasses.removeElement(classe);
          setClasse(null);
          message("La classe '" + classe.getNom() + "' a été correctement retirée...", MESSAGE_INFORMATION);
        } else {
          message("Le fichier " + fichier.getAbsolutePath() + " ne peut être supprimé...", MESSAGE_ERREUR);
        }   
      } else {
        message("Le fichier " + fichier.getAbsolutePath() + " ne peut être supprimé...", MESSAGE_ERREUR);
      }
    }
  }

  public void editerEleve(Eleve eleve) {
    _eleve = eleve;
		setPanneau("PanelEleve");  
  }
  
  public void supprimerEleve(Eleve eleve) {
    int rep = JOptionPane.showConfirmDialog(_pFrame, "Êtes-vous sûr de vouloir retirer l'élève '" + eleve.getNom() + " " + eleve.getPrenom() + "' ?", "Suppression",
        JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE);
    if (rep == JOptionPane.YES_OPTION) {
      modification();
      _classe.remove(eleve);
      _pClasse.updateListe();
    }
  }

  public void modification() {
    modification(_classe);
  }
  
  public void modification(Classe classe) {
    modification(classe, true);
  }
  
  public void modification(Classe classe, boolean oui) {
    if (oui) {
      if (!_lesClassesModifiees.contains(classe)) {
        _lesClassesModifiees.add(classe);
      }
    } else {
      if (_lesClassesModifiees.contains(classe)) {
        _lesClassesModifiees.remove(classe);
      }
    }
  }
  
  public Vector<Application> getLesApplications() {
    return _lesApplications;
  }
  
  public void setTypeGeneration(int type) {
    _typeGeneration = type;
  }

  public int getTypeGeneration() {
    return _typeGeneration;
  }
	
  public void setTailleCurseur(int i) {
    super.setTailleCurseur(i);
  }

  public int getTailleCurseur() {
    return _tailleCurseur;
  }

  public void setAvecTbi(boolean oui) {
    super.setAvecTbi(oui);
  }

  public boolean getAvecTbi() {
    return _avecTbi;
  }
  
  public void setLargeurTbi(int v) {
    super.setLargeurTbi(v);
  }

  public int getLargeurTbi() {
    return _largeurTbi;
  }

  public void setHauteurTbi(int v) {
    super.setHauteurTbi(v);
  }

  public int getHauteurTbi() {
    return _hauteurTbi;
  }
  
  public void setXTbi(int v) {
    super.setXTbi(v);
  }

  public int getXTbi() {
    return _xTbi;
  }
  
  public void setYTbi(int v) {
    super.setYTbi(v);
  }

  public int getYTbi() {
    return _yTbi;
  }
  
  @Override
  public boolean sauveConfiguration() {
    /*************************************
     * 0 : TbiGeneralApp.conf    
     * 1 : TbiGeneral.conf
     * 2 : TbiAdminApp.conf     
     * 3 : TbiAdmin.conf
     *************************************/
    setConfiguration("1", _V_TailleSouris, Integer.toString(_tailleCurseur));
    setConfiguration("1", _V_UtilisationTBI, _avecTbi ? "oui" : "non");
    setConfiguration("1", _V_LargeurTBI, Integer.toString(_largeurTbi));    
    setConfiguration("1", _V_HauteurTBI, Integer.toString(_hauteurTbi));
    setConfiguration("1", _V_CoordXTBI, Integer.toString(_xTbi));    
    setConfiguration("1", _V_CoordYTBI, Integer.toString(_yTbi));
    setConfiguration("1", _V_CheminDesClasses, _cheminDesClasses);
    return sauveConfiguration("1"); // fichier TbiGeneral.conf
  }  

  public PanelClasse getPanelClasse() {
    return _pClasse;
  } 

  public PanelEleve getPanelEleve() {
    return _pEleve;
  }
  
  public ItemEleveGen genereItemEleve(Eleve e) {
    ItemEleve item = new ItemEleve(e);
    return item;
  }
  
  public ItemClasseGen genereItemClasse(Classe c) {
    ItemClasse item = new ItemClasse(c);
    return item;
  }
  
  public void chargeLesApplications() throws Exception { // charge les applications 
    message("Chargement des applications...", MESSAGE_INFORMATION, 0);
    if (_chargeurDePlugins == null) {
      _chargeurDePlugins = new ChargeurDeClasse(new URL[]{}, getClass().getClassLoader());
      //_chargeurDePlugins = new ChargeurDeClasse(new URL[]{}, ClassLoader.getSystemClassLoader());
    }
    _lesApplications = new Vector<Application>();
    File[] fichiers = null;
    File rep = new File(".");
    fichiers = rep.listFiles();
    if (fichiers != null) {
      for(int f = 0; f < fichiers.length; f++) {
        if (fichiers[f].isDirectory()) {
          File plugin = new File(fichiers[f], "pluginEvaluations.jar");
          if (plugin.exists()) {
            try {
              Thread.currentThread().yield();
		          URL u = plugin.toURI().toURL();
              _chargeurDePlugins.addURL(u);
      			  JarFile jar = new JarFile(plugin.getAbsolutePath());
      			  Enumeration enumeration = jar.entries();
      			  while(enumeration.hasMoreElements()){
				        String entree = enumeration.nextElement().toString();
      				  if(entree.toLowerCase().endsWith("manifest.mf")) {
      				    String strManifest = "jar:" + u + "!/" + entree; 
                  URL urlManifest = new URL(strManifest);
                  InputStream isManifest = urlManifest.openStream();      				  
                  BufferedReader readerManifest = new BufferedReader(new InputStreamReader(isManifest));
                  String ligne = "";
                  while ((ligne = readerManifest.readLine()) != null) {
                    if (ligne.trim().toLowerCase().startsWith("main-class:")) {
				              StringTokenizer st = new StringTokenizer(ligne.trim(), ":");
				              st.nextToken();
				              Application app = new Application(fichiers[f].getName(), st.nextToken().trim(), fichiers[f]);
                      app.setClasseSelectionnee(getClasseSelectionnee());
                      app.setElevesSelectionnes(getElevesSelectionnes());
                      getPluginInstance(app); // charge le plugin et charge les évaluations
                      _lesApplications.addElement(app);
                    }
          			  }
			            readerManifest.close(); 
                }
              }
            } catch (Exception e) {
              message(e, MESSAGE_AVERTISSEMENT);      
            }
          }
        }
      }
    }
    if (_lesApplications.size() > 0) 
      message("Chargement des applications : " + _lesApplications.size() + " trouvée" + (_lesApplications.size() > 1 ? "s" : "") + " !", MESSAGE_INFORMATION, 3000);
    else 
      message("Chargement des applications : aucune application trouvée...", MESSAGE_AVERTISSEMENT, 3000);
  }
  
  public PluginEvaluations getPluginInstance(Application app) {
    if (app.getInstance() == null) {
      try {
        Class<?> pluginClass = _chargeurDePlugins.findClass(app.getCheminMainClasse());
        Constructor<?>[] ctrs = pluginClass.getConstructors();
        PluginEvaluations pluginInstance = (PluginEvaluations)(ctrs[0].newInstance());
        pluginInstance.init(this, app);  
        app.setInstance(pluginInstance);
      } catch(Exception e) {
        e.printStackTrace();
      }
    }      
    return app.getInstance();
  }
  
  public void majPlugin(PluginEvaluations plugin) {
    _pSelectApplications.majPlugin(plugin);
  }
  
  @Override
  public String getCheminDesEvaluations() {
    return _cheminDesClasses + File.separator + "evaluations";
  }

  private Application getApplicationByRepertoire(File rep) {
    for(int a = 0; a < _lesApplications.size(); a++)
      if (_lesApplications.elementAt(a).getRepertoire().compareTo(rep) == 0)
        return _lesApplications.elementAt(a); 
    return null;
  }
  
  @Override  
  public void chargeLesEvaluations() {
    // ne pas charger les évaluations par défaut, elles le seront dans chargeLesApplications()
  }
  
  @Override
  public Vector<Eleve> getElevesSelectionnes() {
    return _pSelectEleves.getElevesSelectionnes();
  }
    
  @Override
  public Classe getClasseSelectionnee() {
    return _pSelectEleves.getClasseSelectionnee();
  }
  
  public Vector<Application> getApplicationsSelectionnees() {
    return _pSelectApplications.getApplicationsSelectionnees();
  }
  
	public boolean accept(File dir, String file) {
    return file.toLowerCase().endsWith(".pdf");
	}
	
	public static Dimension getPDFDimension(String chaine, com.itextpdf.text.Font police) {
	  BaseFont bf = police.getCalculatedBaseFont(false);
	  int w = bf.getWidth(chaine)/100;
	  int h = bf.getAscent(chaine)/100 + bf.getDescent(chaine)/100;
	  return new Dimension(w,h);
  }

  public void genererPDF() {
   	message("Génération du document PDF en cours...", MESSAGE_INFORMATION,0);
   	String titre = JOptionPane.showInputDialog("Saisir le titre du document : "); 
  	FileDialog fc = new FileDialog(this,"Création du document PDF", FileDialog.SAVE);
		fc.setFilenameFilter(this);
		fc.setVisible(true);
		if (fc.getDirectory() != null) {
			String chemin = fc.getDirectory() + fc.getFile();
			if (!chemin.toLowerCase().endsWith(".pdf"))
				chemin += ".pdf";
      int hauteur = 842; // points : 297 mm // 72 dpi
	   	int largeur = 595; // points : 210 mm // 72 dpi
	   	Vector <Eleve>lesElevesSelectionnes = getElevesSelectionnes();
	   	Classe classeSelectionnee = getClasseSelectionnee();
	   	try {
		    //Document document = new Document(new com.itextpdf.text.Rectangle(largeur, hauteur));
		    Document document = new Document(PageSize.A4, 30, 30, 30, 30);
        PdfWriter writer = PdfWriter.getInstance(document, new FileOutputStream(chemin));
        writer.setPageEvent(new CustomPdfPage(this, titre));
			  PdfDictionary info = writer.getInfo();
        info.put(PdfName.AUTHOR, new PdfString("TbiSuite/TbiAdmin"));
//        info.put(PdfName.PRODUCER, new PdfString("TbiSuite : http://caroline-et-serge.coude.net/"));
        info.put(PdfName.CREATOR, new PdfString("Caroline et Serge COUDÉ http://caroline-et-serge.coude.net/"));
        info.put(PdfName.TITLE, new PdfString(titre));
        info.put(PdfName.SUBJECT, new PdfString("Résultats d'évaluations"));
        document.open();
 		    ObjetPDF oPDF = new ObjetPDF(document, largeur, hauteur, null, null, null, null, 0, 0);
 		    if (_typeGeneration == TYPE_GENERATION_PAR_APPLICATION) {
          Vector<Application> lesApplicationsSelectionnees = getApplicationsSelectionnees();
            for(int a = 0; a < lesApplicationsSelectionnees.size(); a++) {
              oPDF = lesApplicationsSelectionnees.elementAt(a).getInstance().genererPDF(oPDF);
            }    
        }
 		    if (_typeGeneration == TYPE_GENERATION_PAR_ELEVE) {
          for(int e = 0; e < lesElevesSelectionnes.size(); e++) {
            Eleve eleve = lesElevesSelectionnes.elementAt(e);
            // cartouche
		        PdfPTable cartouche = new PdfPTable(new float[] {1f});
 		        cartouche.setWidthPercentage(40);
 		        cartouche.setHorizontalAlignment(com.itextpdf.text.Element.ALIGN_LEFT);
     		    Paragraph pEleve = new Paragraph("Eleve : " + eleve.getNom() + " " + eleve.getPrenom(), FontFactory.getFont(FontFactory.HELVETICA, 12, com.itextpdf.text.Font.BOLD));
         		pEleve.setAlignment(com.itextpdf.text.Element.ALIGN_LEFT);
 		        PdfPCell cEleve = new PdfPCell(pEleve);
            cartouche.addCell(cEleve);
            Paragraph pClasse = new Paragraph("Classe : " + classeSelectionnee.getNom(), FontFactory.getFont(FontFactory.HELVETICA, 10, com.itextpdf.text.Font.BOLD));
     		    pClasse.setAlignment(com.itextpdf.text.Element.ALIGN_LEFT);
 		        PdfPCell cClasse = new PdfPCell(pClasse);
            cartouche.addCell(cClasse);
            document.add(cartouche);
            // les applications
            Vector<Application> lesApplicationsSelectionnees = getApplicationsSelectionnees();
            for(int a = 0; a < lesApplicationsSelectionnees.size(); a++) {
              oPDF = lesApplicationsSelectionnees.elementAt(a).getInstance().genererPDF(oPDF, eleve);
            }
            if (e < lesElevesSelectionnes.size()-1)
              document.newPage();
          }    
        }
  			document.close();
	 		  message("Création du fichier " + chemin + " correctement effectuée...", MESSAGE_INFORMATION);
			} catch(DocumentException de) {
				message("Erreur survenue durant la création du document PDF...", MESSAGE_ERREUR);
			} catch (FileNotFoundException fnfe) {
				message("Erreur lors de la création du fichier " + chemin + "...", MESSAGE_ERREUR);
			} catch (IOException ioe) {
				message("Erreur lors de l'écriture dans le fichier " + chemin + "...", MESSAGE_ERREUR);
			}
    }
  } 
  
  public void sauvegarde(final File repDest) {
    int rep = JOptionPane.showConfirmDialog(_pFrame, "Attention, les fichiers portant les mêmes noms seront écrasés...\nSouhaitez-vous continuer la sauvegarde ?", "Sauvegarde",
              JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE);
    if (rep == JOptionPane.YES_OPTION) {
      message("Sauvegarde en cours...", MESSAGE_INFORMATION);
      (new Thread(new Runnable() {
        public void run() {
          FileChannel in = null; 
          FileChannel out = null; 
          try {
            if (!repDest.exists()) {
              int rep = JOptionPane.showConfirmDialog(_pFrame, "Le répertoire " + repDest.getAbsolutePath() + " n'existe pas.\nSouhaitez-vous le créer ?", "Sauvegarde",
              JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE);
              if (rep == JOptionPane.YES_OPTION) {
                repDest.mkdir();
              } else {
                message("Abandon de la sauvegarde...", MESSAGE_AVERTISSEMENT);
                return;
              }
            }
            // sauvegarde des classes
            File rep = new File(getConfiguration("chemindesclasses"));
            File[] fichiers = rep.listFiles();
            if (fichiers != null) {
              for(int f = 0; f < fichiers.length; f++) {
                if (fichiers[f].isFile() && fichiers[f].getName().toLowerCase().endsWith("-classe.zip")) {
                  in = new FileInputStream(fichiers[f]).getChannel();
                  out = new FileOutputStream(new File(repDest, fichiers[f].getName())).getChannel();
                  in.transferTo(0, in.size(), out);
                  in.close();
                  out.close();
                }
              }
            }
            // sauvegarde des evaluations
            rep = new File(rep, "evaluations");
            fichiers = rep.listFiles();
            if (fichiers != null) {
              for(int f = 0; f < fichiers.length; f++) {
                if (fichiers[f].isDirectory()) { // un répertoire applicatif
                  File[] fichiersEval = fichiers[f].listFiles();
                  if (fichiersEval != null) {
                    File repDestApp = new File(repDest,"evaluations" + File.separator + fichiers[f].getName());
                    if (!repDestApp.exists()) 
                      repDestApp.mkdirs(); // création du rep applicatif de sauvegarde
                    for(int e = 0; e < fichiersEval.length; e++) {
                      if (fichiersEval[e].isFile() && fichiersEval[e].getName().toLowerCase().endsWith("-eval.xml")) {
                        in = new FileInputStream(fichiersEval[e]).getChannel();
                        out = new FileOutputStream(new File(repDestApp, fichiersEval[e].getName())).getChannel();
                        in.transferTo(0, in.size(), out);
                        in.close();
                        out.close();
                      }
                    }
                  }
                }
              }
            }
            SwingUtilities.invokeLater(new Runnable() {
              public void run() {
                message("Sauvegarde correctement effectuée !", MESSAGE_INFORMATION);
              }
            });
          } catch(Exception e) {
            message("Erreur durant la sauvegarde : " + e.toString() + "...", MESSAGE_ERREUR);
          }
        };
      })).start();
    } else {
      message("Abandon de la sauvegarde...", MESSAGE_AVERTISSEMENT);
    }
  } 
  
  public void restauration(final File repSource) {
    int rep = JOptionPane.showConfirmDialog(_pFrame, "Attention, les fichiers portant les mêmes noms seront écrasés...\nSouhaitez-vous continuer la restauration ?", "Restauration",
              JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE);
    if (rep == JOptionPane.YES_OPTION) {
      message("Restauration en cours...", MESSAGE_INFORMATION);
      (new Thread(new Runnable() {
        public void run() {
          FileChannel in = null; 
          FileChannel out = null; 
          try {
            if (!repSource.exists()) {
              message("Le répertoire de la sauvegarde n'existe pas... Abandon de la restauration...", MESSAGE_AVERTISSEMENT);
              return;
            }
            // restauration des classes
            File repDest = new File(getConfiguration("chemindesclasses"));
            File[] fichiers = repSource.listFiles();
            if (fichiers != null) {
              for(int f = 0; f < fichiers.length; f++) {
                if (fichiers[f].isFile() && fichiers[f].getName().toLowerCase().endsWith("-classe.zip")) {
                  in = new FileInputStream(fichiers[f]).getChannel();
                  out = new FileOutputStream(new File(repDest, fichiers[f].getName())).getChannel();
                  in.transferTo(0, in.size(), out);
                  in.close();
                  out.close();
                }
              }
            }
            // restauration des evaluations
            File repSourceEval = new File(repSource, "evaluations");
            fichiers = repSourceEval.listFiles();
            if (fichiers != null) {
              for(int f = 0; f < fichiers.length; f++) {
                if (fichiers[f].isDirectory()) { // un répertoire applicatif
                  File[] fichiersEval = fichiers[f].listFiles();
                  if (fichiersEval != null) {
                    File repDestApp = new File(repDest,"evaluations" + File.separator + fichiers[f].getName());
                    if (!repDestApp.exists()) 
                      repDestApp.mkdirs(); // création du rep applicatif de sauvegarde
                    for(int e = 0; e < fichiersEval.length; e++) {
                      if (fichiersEval[e].isFile() && fichiersEval[e].getName().toLowerCase().endsWith("-eval.xml")) {
                        in = new FileInputStream(fichiersEval[e]).getChannel();
                        out = new FileOutputStream(new File(repDestApp, fichiersEval[e].getName())).getChannel();
                        in.transferTo(0, in.size(), out);
                        in.close();
                        out.close();
                      }
                    }
                  }
                }
              }
            }
            SwingUtilities.invokeLater(new Runnable() {
              public void run() {
                message("Restauration correctement effectuée !", MESSAGE_INFORMATION);
              }
            });
          } catch(Exception e) {
            message("Erreur durant la restauration : " + e.toString() + "...", MESSAGE_ERREUR);
          }
        };
      })).start();
    } else {
      message("Abandon de la restauration...", MESSAGE_AVERTISSEMENT);
    }
  }   
  
 	public static void main(String[] args) {
 	  try {
      new TbiAdmin("TbiAdmin.log");
	  } catch (Exception e) {
	   e.printStackTrace();
      Log.logStatic(e);
    }
	}

}