/*
Copyright Caroline et Serge COUDÉ, (01/2012) 

serge.coude@gmail.com caroline.coude@laposte.net

Ce logiciel est un programme informatique faisant partie d'une suite de 
logiciels 'TbiSuite' à destination des enseignants et des élèves usagés des 
ordinateurs et Tableau Numérique Interactif / Tableau Blanc Interactif. 

Ce logiciel est régi par la licence CeCILL v2 soumise au droit français et
respectant les principes de diffusion des logiciels libres. Vous pouvez
utiliser, modifier et/ou redistribuer ce programme sous les conditions
de la licence CeCILL v2 telle que diffusée par le CEA, le CNRS et l'INRIA 
sur le site "http://www.cecill.info".

En contrepartie de l'accessibilité au code source et des droits de copie,
de modification et de redistribution accordés par cette licence, il n'est
offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
seule une responsabilité restreinte pèse sur l'auteur du programme,  le
titulaire des droits patrimoniaux et les concédants successifs.

A cet égard  l'attention de l'utilisateur est attirée sur les risques
associés au chargement,  à l'utilisation,  à la modification et/ou au
développement et à la reproduction du logiciel par l'utilisateur étant 
donné sa spécificité de logiciel libre, qui peut le rendre complexe à 
manipuler et qui le réserve donc à des développeurs et des professionnels
avertis possédant  des  connaissances  informatiques approfondies.  Les
utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
logiciel à leurs besoins dans des conditions permettant d'assurer la
sécurité de leurs systèmes et ou de leurs données et, plus généralement, 
à l'utiliser et l'exploiter dans les mêmes conditions de sécurité. 

Le fait que vous puissiez accéder à cet en-tête signifie que vous avez 
pris connaissance de la licence CeCILL v2, et que vous en avez accepté les
termes.

*/

package net.sergecoude.TbiGeneral;

import java.awt.*;
import javax.swing.*;
import java.awt.event.*;
import net.sergecoude.util.*;
import java.io.*;
import java.awt.image.*;
import javax.imageio.*;
import java.net.*;
import java.util.zip.*;
import java.util.*;
import net.n3.nanoxml.*; 

public class Eleve implements IObjetAvecItem {

    private final static long serialVersionUID = 1L;

    private TbiApp _app;
    private JPanelTranslucide _itemForm = null;
    
    private String _id = "";
    private String _nom = "";
    private String _prenom = "";
    private String _nomUsuel = "";
    private int _dateNaissance;
    private Niveau _niveau = null;    
    private boolean _sexe = true;  // true : garçon, false : fille
    private String _cheminImage = "";
    private String _cheminAudio =  "";
    private ItemEleveGen _itemEleve = null;
    private Classe _classe = null;
    private boolean _estEnregistre = false;
    private boolean _estPresent = true;
    
    public static int NOM_ASC = 0;
    public static int NOM_DESC = 1; 
    public static int PRENOM_ASC = 2;
    public static int PRENOM_DESC = 3;
    public static int DNAISSANCE_ASC = 4;
    public static int DNAISSANCE_DESC = 5;
    public static int SEXE_ASC = 6; 
    public static int SEXE_DESC = 7; 
    public static int  NIVEAU_ASC = 8;
    public static int NIVEAU_DESC = 9;
    public static int NOMUSUEL_ASC = 10;
    public static int NOMUSUEL_DESC = 11;
    public static int PRESENT_ASC = 12;
    public static int PRESENT_DESC = 13; 
    private static ArrayList<ParametreDeTri> _lesParametresDeTri;
        
  public Eleve(TbiApp app, Classe classe) {
    _app = app;
    _classe = classe;
    _nom = "";
    _prenom = "";
    _nomUsuel = "";
    _id = "";
    _dateNaissance = 21000101;
    _sexe = true;
    _cheminImage = "";
    _cheminAudio = "";
    _itemEleve = _app.genereItemEleve(this);
  }
  
  public Eleve(TbiApp app, Classe classe, String id) {
    this(app, classe);
    setId(id);
  }
   
  public Eleve(TbiApp app, Classe classe, String id, String nom) {
    this(app, classe, id);
    setNom(nom);
  }
  
  public Eleve(TbiApp app, Classe classe, String id, String nom, String prenom, String nomUsuel) {
    this(app, classe, id, nom);
    setPrenom(prenom);
    setNomUsuel(nomUsuel);
  }
  
  public Eleve(TbiApp app, Classe classe, String id, String nom, String prenom, String nomUsuel, int date) {
    this(app, classe, id, nom, prenom, nomUsuel);
    setDateNaissance(date);
  }
  
  public Eleve(TbiApp app, Classe classe, String id, String nom, String prenom, String nomUsuel, boolean sexe) {
    this(app, classe, id, nom, prenom, nomUsuel);
    setSexe(sexe);
  }
  
  public Eleve(TbiApp app, Classe classe, String id, String nom, String prenom, String nomUsuel, int date, boolean sexe) {
    this(app, classe, id, nom, prenom, nomUsuel);
    setDateNaissance(date);
    setSexe(sexe);
  }
  
  public Eleve(TbiApp app, Classe classe, String id, String nom, String prenom, String nomUsuel, int date, boolean sexe, String cheminImage) {
    this(app, classe, id, nom, prenom, nomUsuel, date);
    setSexe(sexe);
    setCheminImage(cheminImage);
  }

  public Eleve(TbiApp app, Classe classe, String id, String nom, String prenom, String nomUsuel, int date, boolean sexe, String cheminImage, String cheminAudio) {
    this(app, classe, id, nom, prenom, nomUsuel, date, sexe, cheminImage);
    setCheminAudio(cheminAudio);
  }

  public Eleve(TbiApp app, Classe classe, String id, String nom, String prenom, String nomUsuel, int date, boolean sexe, String cheminImage, String cheminAudio, Niveau n) {
    this(app, classe, id, nom, prenom, nomUsuel, date, sexe, cheminImage, cheminAudio);
    setNiveau(n);
  }
  
  public Eleve(TbiApp app, Classe classe, IXMLElement xml) {
    this(app, classe);
    IXMLElement xId = xml.getFirstChildNamed("id");
    IXMLElement xNom = xml.getFirstChildNamed("nom");
		IXMLElement xPrenom = xml.getFirstChildNamed("prenom");
		IXMLElement xNomUsuel = xml.getFirstChildNamed("nomusuel");
    IXMLElement xSexe = xml.getFirstChildNamed("sexe");
    IXMLElement xPresent = xml.getFirstChildNamed("present");
    IXMLElement xDateNaissance = xml.getFirstChildNamed("datenaissance");
    IXMLElement xNiveau = xml.getFirstChildNamed("niveau");
    IXMLElement xPhoto = xml.getFirstChildNamed("photo");
    IXMLElement xAudio = xml.getFirstChildNamed("audio");
    if (xId != null)
      setId(xId.getContent());
    if (xNom != null)
      setNom(xNom.getContent());
    if (xPrenom != null)
      setPrenom(xPrenom.getContent());
    if (xNomUsuel != null)
      setNomUsuel(xNomUsuel.getContent());
    if (xSexe != null)
      if (xSexe.getContent().toLowerCase().equals("m"))
        setSexe(true);
      else 
        setSexe(false);
    if (xPresent != null)
      if (xPresent.getContent().equals("1"))
        setPresent(true);
      else 
        setPresent(false);
    if (xDateNaissance != null) 
	     setDateNaissance(Integer.parseInt(xDateNaissance.getContent()));
	  if (xNiveau != null)
	     setNiveau(_app.getNiveau(xNiveau.getContent()));
	  if (xPhoto != null && xPhoto.getContent() != null && !xPhoto.getContent().equals("")) 
      setCheminImage("file:" + xPhoto.getContent());
	  if (xAudio != null && xAudio.getContent() != null && !xAudio.getContent().equals("")) 
      setCheminAudio("file:" + xAudio.getContent());
    _estEnregistre = true;
  }
  
  public TbiApp getApplication() {
    return _app;
  }
  
  public Classe getClasse() {
    return _classe;
  }

  public void setNom(String nom) {
    if (nom == null)
      _nom = "";
    else 
      _nom = nom;
    if (_itemEleve != null)  
      _itemEleve.setNom(_nom);
  }
  
  public String getNom() {
    return _nom;
  }

  public void setId(String id) {
    _id = id;
  }
  
  public String getId() {
    return _id;
  }
  
  public boolean estEnregistre() {
    return _estEnregistre;
  }
  
  public void setPresent(boolean present) {
    _estPresent = present;
    if (_itemEleve != null)  
      _itemEleve.setPresent(present);
  }
  
  public boolean estPresent() {
    return _estPresent;
  }

  public void setPrenom(String prenom) {
    if (prenom == null)
      _prenom = "";
    else 
      _prenom = prenom;
    if (_itemEleve != null)  
      _itemEleve.setPrenom(_prenom);
  }
  
  public String getPrenom() {
    return _prenom;
  }
  
  public void setNomUsuel(String nomUsuel) {
    if (nomUsuel == null)
      _nomUsuel = "";
    else 
      _nomUsuel = nomUsuel;
    if (_itemEleve != null)  
      _itemEleve.setNomUsuel(_nomUsuel);
  }
  
  public String getNomUsuel() {
    return _nomUsuel;
  }
  
  public void setDateNaissance(int date) {
    _dateNaissance = date;
    if (_itemEleve != null)    
      _itemEleve.setDateNaissance(getDateNaissanceFormate());
  }
  
  public int getDateNaissance() {
    return _dateNaissance;
  }

  public String getDateNaissanceFormate() {
      String dn = Integer.toString(getDateNaissance());
      return dn.substring(6) + "/" + dn.substring(4,6) + "/" + dn.substring(0,4);
  }
  
  public void setSexe(boolean sexe) {
    _sexe = sexe;
    if (_itemEleve != null)    
      _itemEleve.setSexe(sexe ? "M" : "F");
  }
  
  public boolean getSexe() {
    return _sexe;
  }
  
  public void setCheminImage(String chemin) {
    _cheminImage = chemin;
    if (_itemEleve != null)  
      _itemEleve.setImage(_classe.getPhoto(this));
  }
  
  public String getCheminImage() {
    if (_cheminImage.indexOf("file:") >= 0) {
      return _cheminImage.substring(_cheminImage.indexOf("file:")+5);
    } else {
      return _cheminImage;
    }
  }

  public void setCheminAudio(String chemin) {
    _cheminAudio = chemin;
    if (_itemEleve != null)  
      _itemEleve.setAudio(_classe.getAudio(this));
  }
  
  public String getCheminAudio() {
    if (_cheminAudio.indexOf("file:") >= 0) {
      return _cheminAudio.substring(_cheminAudio.indexOf("file:")+5);
    } else {
      return _cheminAudio;
    }
  }
  
  public void setNiveau(Niveau n) {
    _niveau = n;
    if (_itemEleve != null) {
      if (_niveau != null)
        _itemEleve.setNiveau(_niveau.getNom());
      else
        _itemEleve.setNiveau("n.d.");
    }   
  }
  
  public Niveau getNiveau() {
    return _niveau;
  }
  
  public String getInterneCheminImage() {
    return _cheminImage;
  }
  
  public String getInterneCheminAudio() {
    return _cheminAudio;
  }
  
  public String toString() {
    return _prenom + " " + _nom + " (" + (_sexe ? "M" : "F") + " / " + _dateNaissance + ")";
  }

  public ItemEleveGen getItemEleve() {
    return _itemEleve;
  }
  
  public JPanel getItem() {
    return getItemEleve();
  }  
  
  public String toXml() {
    String xml = "<eleve>";
    xml += "<id>" + _id + "</id>";
    xml += "<nom>" + _nom + "</nom>";
    xml += "<prenom>" + _prenom + "</prenom>";
    xml += "<nomusuel>" + _nomUsuel + "</nomusuel>";
    xml += "<sexe>" + (_sexe ? "M" : "F" ) + "</sexe>";
    xml += "<present>" + (_estPresent ? "1" : "0" ) + "</present>";
    xml += "<datenaissance>" + Integer.toString(_dateNaissance) + "</datenaissance>";
    xml += "<niveau>" + (getNiveau() == null ? "" :  getNiveau().getIdx()) + "</niveau>";
    xml += "<photo>" + transformeChemin(_cheminImage) + "</photo>";
    xml += "<audio>" + transformeChemin(_cheminAudio) + "</audio>";
    xml += "</eleve>";
    return xml;
  }
  
  public String transformeChemin(String chemin) {
    if (chemin.indexOf("file:") >= 0) {
      return chemin.substring(chemin.indexOf("file:")+5);
    } else {
      return chemin.substring(chemin.lastIndexOf(File.separator)+1);
    }
  }
  
  public static ArrayList<ParametreDeTri> getParametresDeTri() {
    if (_lesParametresDeTri == null) {
      _lesParametresDeTri = new ArrayList<ParametreDeTri>();
      _lesParametresDeTri.add(new ParametreDeTri(NOM_ASC, "Nom A->Z"));
      _lesParametresDeTri.add(new ParametreDeTri(NOM_DESC, "Nom Z->A"));
      _lesParametresDeTri.add(new ParametreDeTri(PRENOM_ASC, "Prénom A->Z"));
      _lesParametresDeTri.add(new ParametreDeTri(PRENOM_DESC, "Prénom Z->A"));
      _lesParametresDeTri.add(new ParametreDeTri(SEXE_ASC, "Sexe M,F"));
      _lesParametresDeTri.add(new ParametreDeTri(SEXE_DESC, "Sexe F,M"));
      _lesParametresDeTri.add(new ParametreDeTri(DNAISSANCE_DESC, "Jeune->Agé"));
      _lesParametresDeTri.add(new ParametreDeTri(DNAISSANCE_ASC, "Agé->Jeune"));
      _lesParametresDeTri.add(new ParametreDeTri(NIVEAU_ASC, "Niveau asc"));
      _lesParametresDeTri.add(new ParametreDeTri(NIVEAU_DESC, "Niveau desc"));
      _lesParametresDeTri.add(new ParametreDeTri(NOMUSUEL_ASC, "Nom usuel A->Z"));
      _lesParametresDeTri.add(new ParametreDeTri(NOMUSUEL_DESC, "Nom usuel Z->A"));
      _lesParametresDeTri.add(new ParametreDeTri(PRESENT_ASC, "Présent Non,Oui"));
      _lesParametresDeTri.add(new ParametreDeTri(PRESENT_DESC, "Présent Oui,Non"));
    }
    return _lesParametresDeTri;
  } 

  public static Comparator<Eleve> getComparator(ArrayList<ParametreDeTri> paramTri) {
    return new EleveComparator(paramTri);
  }
  
  private static class EleveComparator implements Comparator<Eleve> {
        private ArrayList<ParametreDeTri> _parametres;

        private EleveComparator(ArrayList<ParametreDeTri> parametres) {
            _parametres = parametres;
        }

        public int compare(Eleve e1, Eleve e2) {
            int comparaison = 0;
            for(int i = 0; i < _parametres.size(); i++ ) {
                int idparam = _parametres.get(i).getId();
                if (idparam == Eleve.NOM_ASC)
                  comparaison = e1.getNom().compareTo(e2.getNom());
                if (idparam == Eleve.NOM_DESC)
                  comparaison = e2.getNom().compareTo(e1.getNom());
                if (idparam == Eleve.PRENOM_ASC)
                  comparaison = e1.getPrenom().compareTo(e2.getPrenom());                
                if (idparam == Eleve.PRENOM_DESC)
                  comparaison = e2.getPrenom().compareTo(e1.getPrenom());
                if (idparam == Eleve.NOMUSUEL_ASC)
                  comparaison = e1.getNomUsuel().compareTo(e2.getNomUsuel());
                if (idparam == Eleve.NOMUSUEL_DESC)
                  comparaison = e2.getNomUsuel().compareTo(e1.getNomUsuel());
                if (idparam == Eleve.DNAISSANCE_ASC)
                    comparaison = e1.getDateNaissance() - e2.getDateNaissance();                
                if (idparam == Eleve.DNAISSANCE_DESC)
                    comparaison = e2.getDateNaissance() - e1.getDateNaissance();                
                if (idparam == Eleve.SEXE_ASC)
                    if (e1.getSexe() == e2.getSexe()) comparaison = 0;
                    else if (e1.getSexe()) comparaison = -1;
                    else comparaison = 1;
                if (idparam == Eleve.SEXE_DESC)
                    if (e1.getSexe() == e2.getSexe()) comparaison = 0;
                    else if (e1.getSexe()) comparaison = 1;
                    else comparaison = -1;
                if (idparam == Eleve.NIVEAU_ASC)
                    comparaison = e1.getNiveau().getIdx() - e2.getNiveau().getIdx();
                if (idparam == Eleve.NIVEAU_DESC)                
                    comparaison = e2.getNiveau().getIdx() - e1.getNiveau().getIdx();
                if (idparam == Eleve.PRESENT_ASC)
                    if (e1.estPresent() == e2.estPresent()) comparaison = 0;
                    else if (e1.estPresent()) comparaison = 1;
                    else comparaison = -1;
                if (idparam == Eleve.PRESENT_DESC)
                    if (e1.estPresent() == e2.estPresent()) comparaison = 0;
                    else if (e1.estPresent()) comparaison = -1;
                    else comparaison = 1;
                if (comparaison != 0) return comparaison;
            }
            return 0;
        }
    }  

}
 